# -*- coding: utf-8 -*-

__all__ = ['DNAFULL', 'BLOSUM62']

_NUCL_LETTERS = [
    b'A', b'T', b'G', b'C', b'U',        # non-ambiguous letters
    b'R', b'Y', b'S', b'W', b'K', b'M',   # 2-base ambiguity
    b'B', b'D', b'H', b'V',             # 3-base ambiguity
    b'N',                            # 4-base ambiguity
]

_NUCL_SCORES = {
     #     A   T   G   C   U   R   Y   S   W   K   M   B   D   H   V   N
    b'A': [ 5, -4, -4, -4, -4,  1, -4, -4,  1, -4,  1, -4, -1, -1, -1, -2],
    b'T': [-4,  5, -4, -4,  5, -4,  1, -4,  1,  1, -4, -1, -1, -1, -4, -2],
    b'G': [-4, -4,  5, -4, -4,  1, -4,  1, -4,  1, -4, -1, -1, -4, -1, -2],
    b'C': [-4, -4, -4,  5, -4, -4,  1,  1, -4, -4,  1, -1, -4, -1, -1, -2],
    b'U': [-4,  5, -4, -4,  5, -4,  1, -4,  1,  1, -4, -1, -1, -1, -4, -2],

    b'R': [ 1, -4,  1, -4, -4, -1, -4, -2, -2, -2, -2, -3, -1, -3, -1, -1],
    b'Y': [-4,  1, -4,  1,  1, -4, -1, -2, -2, -2, -2, -1, -3, -1, -3, -1],
    b'S': [-4, -4,  1,  1, -4, -2, -2, -1, -4, -2, -2, -1, -3, -3, -1, -1],
    b'W': [ 1,  1, -4, -4,  1, -2, -2, -4, -1, -2, -2, -3, -1, -1, -3, -1],
    b'K': [-4,  1,  1, -4,  1, -2, -2, -2, -2, -1, -4, -1, -1, -3, -3, -1],
    b'M': [ 1, -4, -4,  1, -4, -2, -2, -2, -2, -4, -1, -3, -3, -1, -1, -1],

    b'B': [-4, -1, -1, -1, -1, -3, -1, -1, -3, -1, -3, -1, -2, -2, -2, -1],
    b'D': [-1, -1, -1, -4, -1, -1, -3, -3, -1, -1, -3, -2, -1, -2, -2, -1],
    b'H': [-1, -1, -4, -1, -1, -3, -1, -3, -1, -3, -1, -2, -2, -1, -2, -1],
    b'V': [-1, -4, -1, -1, -4, -1, -3, -1, -3, -3, -1, -2, -2, -2, -1, -1],

    b'N': [-2, -2, -2, -2, -2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1],
}

# NOTE: This relies on the ordering of _NUCL_LETTERS and the lists inside
#       _NUCL_SCORES above.
DNAFULL = {letter: dict(zip(_NUCL_LETTERS, scores))
           for letter, scores in _NUCL_SCORES.items()}

_AA_LETTERS = [
    b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'K', b'L', b'M',
    b'N', b'P', b'Q', b'R', b'S', b'T', b'V', b'W', b'X', b'Y', b'Z', b'*',
]

_BLOSUM62_SCORES = {
     #     A   B   C   D   E   F   G   H   I   K   L   M   N   P   Q   R   S   T   V   W   X   Y   Z   *
    b'A': [ 4, -2,  0, -2, -1, -2,  0, -2, -1, -1, -1, -1, -2, -1, -1, -1,  1,  0,  0, -3,  0, -2, -1, -4],
    b'B': [-2,  4, -3,  4,  1, -3, -1,  0, -3,  0, -4, -3,  3, -2,  0, -1,  0, -1, -3, -4, -1, -3,  1, -4],
    b'C': [ 0, -3,  9, -3, -4, -2, -3, -3, -1, -3, -1, -1, -3, -3, -3, -3, -1, -1, -1, -2, -2, -2, -3, -4],
    b'D': [-2,  4, -3,  6,  2, -3, -1, -1, -3, -1, -4, -3,  1, -1,  0, -2,  0, -1, -3, -4, -1, -3,  1, -4],
    b'E': [-1,  1, -4,  2,  5, -3, -2,  0, -3,  1, -3, -2,  0, -1,  2,  0,  0, -1, -2, -3, -1, -2,  4, -4],
    b'F': [-2, -3, -2, -3, -3,  6, -3, -1,  0, -3,  0,  0, -3, -4, -3, -3, -2, -2, -1,  1, -1,  3, -3, -4],
    b'G': [ 0, -1, -3, -1, -2, -3,  6, -2, -4, -2, -4, -3,  0, -2, -2, -2,  0, -2, -3, -2, -1, -3, -2, -4],
    b'H': [-2,  0, -3, -1,  0, -1, -2,  8, -3, -1, -3, -2,  1, -2,  0,  0, -1, -2, -3, -2, -1,  2,  0, -4],
    b'I': [-1, -3, -1, -3, -3,  0, -4, -3,  4, -3,  2,  1, -3, -3, -3, -3, -2, -1,  3, -3, -1, -1, -3, -4],
    b'K': [-1,  0, -3, -1,  1, -3, -2, -1, -3,  5, -2, -1,  0, -1,  1,  2,  0, -1, -2, -3, -1, -2,  1, -4],
    b'L': [-1, -4, -1, -4, -3,  0, -4, -3,  2, -2,  4,  2, -3, -3, -2, -2, -2, -1,  1, -2, -1, -1, -3, -4],
    b'M': [-1, -3, -1, -3, -2,  0, -3, -2,  1, -1,  2,  5, -2, -2,  0, -1, -1, -1,  1, -1, -1, -1, -1, -4],
    b'N': [-2,  3, -3,  1,  0, -3,  0,  1, -3,  0, -3, -2,  6, -2,  0,  0,  1,  0, -3, -4, -1, -2,  0, -4],
    b'P': [-1, -2, -3, -1, -1, -4, -2, -2, -3, -1, -3, -2, -2,  7, -1, -2, -1, -1, -2, -4, -2, -3, -1, -4],
    b'Q': [-1,  0, -3,  0,  2, -3, -2,  0, -3,  1, -2,  0,  0, -1,  5,  1,  0, -1, -2, -2, -1, -1,  3, -4],
    b'R': [-1, -1, -3, -2,  0, -3, -2,  0, -3,  2, -2, -1,  0, -2,  1,  5, -1, -1, -3, -3, -1, -2,  0, -4],
    b'S': [ 1,  0, -1,  0,  0, -2,  0, -1, -2,  0, -2, -1,  1, -1,  0, -1,  4,  1, -2, -3,  0, -2,  0, -4],
    b'T': [ 0, -1, -1, -1, -1, -2, -2, -2, -1, -1, -1, -1,  0, -1, -1, -1,  1,  5,  0, -2,  0, -2, -1, -4],
    b'V': [ 0, -3, -1, -3, -2, -1, -3, -3,  3, -2,  1,  1, -3, -2, -2, -3, -2,  0,  4, -3, -1, -1, -2, -4],
    b'W': [-3, -4, -2, -4, -3,  1, -2, -2, -3, -3, -2, -1, -4, -4, -2, -3, -3, -2, -3, 11, -2,  2, -3, -4],
    b'X': [ 0, -1, -2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1,  0,  0, -1, -2, -1, -1, -1, -4],
    b'Y': [-2, -3, -2, -3, -2,  3, -3,  2, -1, -2, -1, -1, -2, -3, -1, -2, -2, -2, -1, 2, - 1,  7, -2, -4],
    b'Z': [-1,  1, -3,  1,  4, -3, -2,  0, -3,  1, -3, -1,  0, -1,  3,  0,  0, -1, -2, -3, -1, -2,  4, -4],
    b'*': [-4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4,  1],
}

# NOTE: This relies on the ordering of _AA_LETTERS and the lists inside
#       _BLOSUM62_SCORES above.
BLOSUM62 = {letter: dict(zip(_AA_LETTERS, scores))
            for letter, scores in _BLOSUM62_SCORES.items()}